package com.tripto.bjlt;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tripto.bjlt.entity.CacheDuration;
import com.tripto.bjlt.entity.PolicyEntity;
import com.tripto.bjlt.utils.RedisCacheUtil;

/**
 * 控制器，管理
 * @ClassName: ControManage 
 * @Description: TODO(这里用一句话描述这个类的作用) 
 * @author zhangqifei 
 * @date 2018年3月28日 下午2:18:54
 */
public class ControManage {

	//1.航空公司 KEY=AK(航司) VALUE=[数据源，数据源编码，生单地址]
	public static Map<String, String[]> airwaysMap = new HashMap<String, String[]>();
	
	//2.平台Map KEY=平台编码  VALUE=航司数据源,默认乘客数,是否支持急速出票
	public static Map<String,Map<String,Integer[]>> platformMap = new HashMap<String,Map<String,Integer[]>>();
	
	//3.城市代码转换  KEY=AK(航司) VALUE=Map<城市代码,对应航司代码>
	public static Map<String,String> cityToAirplainMap = new HashMap<String,String>();
	
	//4.全局日期配置数据 KEY=平台代码_航司代码    提前天数
	public static Map<String,Integer> wholeDateMap = new HashMap<String,Integer>(); 
	
	//5.全局调价  KEY=平台代码_航司代码    VALUE={KEY=[起始日期,结束日期],VALUE={KEY=产品类型,VALUSE=[成人调价,儿童调价]}}
	public static Map<String,Map<String,Map<String,Integer[]>>> wholePriceMap = new HashMap<String,Map<String,Map<String,Integer[]>>>();
	
	//6.航司航线数据 KEY=航司代码_出发城市三字码_抵达城市三字码 VALUE=出发国家（没有什么嘛用）代码
	public static Map<String,String> airwaysRouteMap = new HashMap<String,String>();
	
	//7.航司币种汇率  KEY=适用航司(AK) VALUE= {KEY=外币种_兑换币种 VALUE=最终汇率}
	public static Map<String,Map<String,BigDecimal>> exchangeRateMap = new HashMap<String,Map<String,BigDecimal>>();
	
	//8.银联汇率  KEY=外币种_兑换币种 VALUE=最终汇率
	public static Map<String,BigDecimal> UnionPayExchangeRateMap = new HashMap<String,BigDecimal>();
	
	//9.国家币种  KEY=国家代码_机场代码   VALUE=币种码值
	public static Map<String,String> currencyMap = new HashMap<String,String>();
	
	//10.政策 KEY = 平台代码_航司代码_出发城市代码_抵达城市代码  VALUE=List<政策对象信息>
	public static Map<String,List<PolicyEntity>> policyMap = new HashMap<String,List<PolicyEntity>>();

	//11.全局调价(平台)  KEY=平台代码    VALUE={KEY=产品类型,VALUSE=[成人调价,儿童调价]}
	public static Map<String,Map<String,Integer[]>> WholePlatformPrice = new HashMap<String,Map<String,Integer[]>>();
	
	//12.城市三字码对应币种 KEY=城市三字码  VALUE=币种
    public static Map<String,String> cityCurrency = new HashMap<>();
		
    //13.缓存规则  Map KEY=航司_时间 VALUE={KEY=类型+出发城市+到达城市,VALUE=缓存时长}
    public static Map<String,Map<String,Integer>> cacheDuration = new HashMap<>();
    
	//当前时间  20180406
	public static int currentDate = 20180406;
	
	/**
	 * 创建与初始化Contro
	 * @param codeNum
	 * @param itemJSONObj
	 */
	public static void setContro(int codeNum,JSONObject itemJSONObj) {
		if(codeNum == 1) {
			airwaysMap = new HashMap<String, String[]>();
			for (String key : itemJSONObj.keySet()) {
				JSONArray array = itemJSONObj.getJSONArray(key);
				String[] info = new String[array.size()];
				for(int i = 0 ; i< array.size() ; i++) {
					info[i] = array.get(i).toString();
				}
				airwaysMap.put(key, info);
			}
		}else if(codeNum ==2) {
			platformMap = new HashMap<>();
			for (String key : itemJSONObj.keySet()) {
				JSONObject jsonObj = itemJSONObj.getJSONObject(key);
				Map<String,Integer[]> map = new HashMap<>();
				for (String jkey : jsonObj.keySet()) {
					JSONArray array = jsonObj.getJSONArray(jkey);
					Integer[] infos = new Integer[array.size()];
					for(int i = 0 ; i< infos.length;i++) {
						infos[i] = array.getInteger(i);
					}
					map.put(jkey, infos);
				}
				platformMap.put(key, map);
			}
		}else if(codeNum ==3) {
			cityToAirplainMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==4) {
			wholeDateMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==5) {
			wholePriceMap = new HashMap<>();
			//KEY=平台代码_航司代码    VALUE={KEY=[起始日期,结束日期],VALUE={KEY=产品类型,VALUSE=[成人调价,儿童调价]}}
			//Map<String,Map<String,Map<String,Integer[]>>>
			for (String key : itemJSONObj.keySet()) {
				JSONObject dateJson = itemJSONObj.getJSONObject(key);
				Map<String,Map<String,Integer[]>> dateMap = new HashMap<>();
				for(String date : dateJson.keySet()){
					Map<String,Integer[]> typeMap = new HashMap<>();
					JSONObject typeJson = dateJson.getJSONObject(date);
					for(String type:typeJson.keySet()) {
						JSONArray array = typeJson.getJSONArray(type);
						Integer[] infos = new Integer[array.size()];
						for(int i = 0 ; i < array.size() ; i++) {
							infos[i] = array.getInteger(i);
						}
						typeMap.put(type, infos);
					}
					dateMap.put(date, typeMap);
				}
				wholePriceMap.put(key, dateMap);
			}
		}else if(codeNum ==6) {
			airwaysRouteMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==7) {
			exchangeRateMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==8) {
			UnionPayExchangeRateMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==9) {
			currencyMap = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==10) {
			PolicySync policySync = new PolicySync();
			policySync.start();
		}else if(codeNum ==11) {
			WholePlatformPrice = new HashMap<>();
			//KEY=平台代码    VALUE=List<VALUE={KEY=产品类型,VALUSE=[成人调价,儿童调价]}>
			//Map<String,Map<String,Integer[]>>
			for (String key : itemJSONObj.keySet()) {
				Map<String,Integer[]> typeMap = new HashMap<>();
				JSONObject typeJson = itemJSONObj.getJSONObject(key);
				for(String type:typeJson.keySet()) {
					JSONArray array = typeJson.getJSONArray(type);
					Integer[] infos = new Integer[array.size()];
					for(int i = 0 ; i < array.size() ; i++) {
						infos[i] = array.getInteger(i);
					}
					typeMap.put(type, infos);
				}
				WholePlatformPrice.put(key, typeMap);
			}
		}else if(codeNum ==12) {
			cityCurrency = JSONObject.toJavaObject(itemJSONObj, Map.class);
		}else if(codeNum ==13) {
			//这里需要特殊处理一下
			if(itemJSONObj.getBoolean("msg")) {
				JSONArray array = itemJSONObj.getJSONArray("cacheDuration");
				List<CacheDuration> list = new ArrayList<>();
				for (int i = 0; i < array.size(); i++) {
					CacheDuration cd = JSONObject.toJavaObject(array.getJSONObject(i), CacheDuration.class);
					list.add(cd);
				}
				cacheDuration = getCacheDuration(list);
			}
		}
	}
	
	/**
	 * 获取Contro配置参数
	 * @param codeNum
	 */
	public static JSONObject getContro(int codeNum){
		if(codeNum == 1) {
			return JSONObject.parseObject(JSON.toJSONString(airwaysMap));
		}else if(codeNum ==2) {
			return JSONObject.parseObject(JSON.toJSONString(platformMap));
		}else if(codeNum ==3) {
			return JSONObject.parseObject(JSON.toJSONString(cityToAirplainMap));
		}else if(codeNum ==4) {
			return JSONObject.parseObject(JSON.toJSONString(wholeDateMap));
		}else if(codeNum ==5) {
			return JSONObject.parseObject(JSON.toJSONString(wholePriceMap));
		}else if(codeNum ==6) {
			return JSONObject.parseObject(JSON.toJSONString(airwaysRouteMap));
		}else if(codeNum ==7) {
			return JSONObject.parseObject(JSON.toJSONString(exchangeRateMap));
		}else if(codeNum ==8) {
			return JSONObject.parseObject(JSON.toJSONString(UnionPayExchangeRateMap));
		}else if(codeNum ==9) {
			return JSONObject.parseObject(JSON.toJSONString(currencyMap));
		}else if(codeNum ==10) {
			return JSONObject.parseObject(JSON.toJSONString(policyMap));
		}else if(codeNum ==11) {
			return JSONObject.parseObject(JSON.toJSONString(WholePlatformPrice));
		}else if(codeNum ==12) {
			return JSONObject.parseObject(JSON.toJSONString(cityCurrency));
		}else if(codeNum ==13) {
			return JSONObject.parseObject(JSON.toJSONString(cacheDuration));
		}
		return null;
	}
	
	/**
	 * 拆解缓存数据规则
	 * @param list
	 * @return
	 */
	public static Map<String,Map<String,Integer>> getCacheDuration(List<CacheDuration> list){
		Map<String,String> airMap = new HashMap<String,String>();
		Integer[] date = get365Day();
		Map<String,Map<String,Integer>> cacheDuration = new HashMap<String,Map<String,Integer>>();
		//规则类型1.航司时间段 2.出发城市 3.到达城市  时间计算模型 1.时间段 2.限制天数
		for(CacheDuration cd :list) {
			String airName = cd.getAirName();
			if(!airMap.containsKey(airName)){
				//标识没有这个航司那么添加  并且加载365天
				airMap.put(airName,"");
				for(Integer dt:date) {
					Map<String,Integer> map = new HashMap<>();
					map.put("1", 30*60);
					cacheDuration.put(airName+"_"+dt, map);
				}
			}
			//准备插入值  计算开始下标  
			int index = cd.getStarti();
			if(index < 0)index = 0;
			int endIndex = index + cd.getDays();
			if(date.length < endIndex) endIndex = date.length;
			int caache_type = cd.getCacheType();
			String key = caache_type+"";
			if(caache_type == 2)
				key +=cd.getFromCity();
			if(caache_type == 3)
				key +=cd.getFromCity()+cd.getToCity();
			int duration = cd.getDuration();
			for(int i = index; i < endIndex ; i++) {
				cacheDuration.get(airName+"_"+date[i]).put(key, duration);
			}
		}
		return cacheDuration;
	}
	
	
	/**
	 * 获取当前时间后365的日期数组
	 * @return
	 */
	public static Integer[] get365Day(){
		Integer[] datetime = new Integer[365];
		Date currentDate = new Date();
		SimpleDateFormat df = new SimpleDateFormat("yyyyMMdd");
		Calendar c = Calendar.getInstance();  
        c.setTime(currentDate);  
		for(int i = 0 ; i < datetime.length ;i++) {
			datetime[i] = Integer.valueOf(df.format(c.getTime()));
			c.add(Calendar.DAY_OF_MONTH, 1);
		}
		return datetime;
	}
	
	/**
	 * 计算缓存时效
	 * @param airName
	 * @param fromDate
	 * @param retDate
	 * @return
	 */
	public static int CalculationDuration(String airName,String fromDate,String retDate,String fromCity,String toCity) {
		fromDate = fromDate.replaceAll("-", "");
		if(retDate != null && !retDate.equals("")) retDate = retDate.replaceAll("-", "");
		//判断原则   KEY=3PEKKUL KEY=2PEK KEY=1
		String fkey3 = "3"+fromCity+toCity;
		String fkey2 = "2"+fromCity;
		int dura = 45*60;
		int fromDura = dura;
		int toDura = dura;
		if(fromDate != null && fromDate.length() == 10) {
			Map<String,Integer> from = cacheDuration.get(airName+"_"+fromDate);
			if(from != null && from.size() > 0) {
				if(from.containsKey(fkey3)) {
					fromDura = from.get(fkey3);
				}else if(from.containsKey(fkey2)) {
					fromDura = from.get(fkey2);
				}else if(from.containsKey("1")){
					fromDura = from.get("1");
				}
			}
		}
		//判断是否是往返
		if(toCity != null && toCity.length() == 10) {
			Map<String,Integer> from = cacheDuration.get(airName+"_"+retDate);
			if(from != null && from.size() > 0) {
				String tkey3 = "3"+toCity+fromCity;
				String tkey2 = "2"+toCity;
				if(from.containsKey(tkey3)) {
					fromDura = from.get(tkey3);
				}else if(from.containsKey(tkey2)) {
					fromDura = from.get(tkey2);
				}else if(from.containsKey("1")){
					fromDura = from.get("1");
				}
			}
		}
		if(fromDura < toDura) {
			return fromDura;
		}else {
			return toDura;
		}
	}
	
	/**
	 * 启动加载缓存
	 */
	public static void StartLoad() {
		JSONObject itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_1"));
		ControManage.setContro(1, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_2"));
		ControManage.setContro(2, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_3"));
		ControManage.setContro(3, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_4"));
		ControManage.setContro(4, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_5"));
		ControManage.setContro(5, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_6"));
		ControManage.setContro(6, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_7"));
		ControManage.setContro(7, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_8"));
		ControManage.setContro(8, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_9"));
		ControManage.setContro(9, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_11"));
		ControManage.setContro(11, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_12"));
		ControManage.setContro(12, itemJSONObj);
		itemJSONObj = JSONObject.parseObject(RedisCacheUtil.getRedis("CONTRO_13"));
		ControManage.setContro(13, itemJSONObj);
		ControManage.setContro(10, null);//里面进行加载
	}
	
	public static void main(String[] args) {
		String dt = "20180510";
		System.out.println(dt.substring(0, 4)+"-"+dt.substring(4, 6)+"-"+dt.substring(6, 8));
		
		System.out.println(ControManage.policyMap.get("dfasljkdfjlaskdjg"));
		
	}
}

/**
 * 政策同步
 * @ClassName: PolicySync 
 * @Description: TODO(这里用一句话描述这个类的作用) 
 * @author zhangqifei 
 * @date 2018年4月17日 下午4:00:58
 */
class PolicySync extends Thread{
	@Override
    public void run() {
		ControManage.policyMap = new HashMap<>();
		String[] sr = RedisCacheUtil.GetKeys("CONTRO_10_Policy_*");
    	for (int i = 0; i < sr.length; i++) {
			JSONArray array = JSONArray.parseArray(RedisCacheUtil.getRedis(sr[i]));
			System.out.println(sr[i]+"\t"+array.size());
			for(int j = 0 ; j < array.size() ;j ++) {
				PolicyEntity policy = JSONObject.toJavaObject(array.getJSONObject(j), PolicyEntity.class);
				if(ControManage.policyMap.containsKey(policy.policyKey)) {
					ControManage.policyMap.get(policy.policyKey).add(policy);
				}else {
					List<PolicyEntity> list = new ArrayList<>();
					list.add(policy);
					ControManage.policyMap.put(policy.policyKey, list);
				}
			}
		}
	}
} 
