package com.tripto.bjlt.service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.alibaba.fastjson.JSON;
import com.bjlt.spider.entity.AirlineInfomation;
import com.bjlt.spider.entity.AirlinePriceInfo;
import com.bjlt.spider.entity.RouteInfoEntity;
import com.bjlt.spider.entity.SearchRequestEntity;
import com.bjlt.spider.entity.SearchResponseEntity;
import com.tripto.bjlt.ControManage;
import com.tripto.bjlt.entity.PolicyEntity;
import com.tripto.bjlt.entity.SearchRequestAfterOta;
import com.tripto.bjlt.entity.SearchResultSet;
import com.tripto.bjlt.entity.SearchResultSetSegments;
import com.tripto.bjlt.entity.SearchResultTripto;
import com.tripto.bjlt.utils.SpiderSearchUtils;

/**
 * 查询Service
 * @ClassName: SearchService 
 * @Description: TODO(这里用一句话描述这个类的作用) 
 * @author zhangqifei 
 * @date 2018年4月4日 下午4:40:11
 */
public class SearchService {

	SearchRequestAfterOta afterOta = null;
	
	private String platform;//平台标识
	private String fromDate;//去程时间  20180531
	private String retDate;//返程时间
	private String fromCity;//出发城市三字码代码
	private String toCity;//抵达城市三字码代码
	private String tripType;//航程类型 单程 往返 或者多程 用枚举控制
	private Integer adultNum;//成人数量
	private Integer childNum;//儿童数量
	private int passNum;//通道标识
	private boolean isStartLog = false; //是否启动日志  前期收集请求参数，匹配数据源   发送组装数据记录     记录各个节点的耗时   1.前期组装 2.中间查询  3.后期组装

	/**
     * true  打开缓存  false 关闭缓存
     */
	private boolean openCaching = true;
	
	long startTime = System.currentTimeMillis();    //获取开始时间
	
	private Map<String,Integer[]> dataSource;//可查询航司数据源 航司数据源,默认乘客数,是否支持急速出票 1.是 [{"AK":[3,1]},{"VJ":[3,2]}]
	
	//KEY=产品类型
	private Map<String,Integer[]> platForm;//平台加减价
	//KEY=航司_产品类型
	private Map<String,Integer[]> wholePrice;//航司平台加减价
	
	private List<SearchRequestEntity> searchList;//查询拆分需求
	
	private List<SearchResponseEntity> retSearch;//查询结果
	
	private List<SearchResultSet> searchFromResultSets;//去程查询结果集
	private List<SearchResultSet> searchToResultSets;//返程查询结果集
	
	private List<SearchResultSet> finalResultSets;//最终结果集
	
	private SearchResultTripto resultTripto;//返回对象
	
	public String uuid;//查询UUID
	
	public Map<String,Boolean> conclusion;//查询结论
	
	public List<String> searchKey;
	/**
	 * 初始化构造
	 * @param afterOta
	 */
	public SearchService(SearchRequestAfterOta afterOta) {
		this.afterOta = afterOta;
		passNum = afterOta.getPassNum();
		platform = afterOta.getCid();
		fromCity = afterOta.getFromCity();
		toCity = afterOta.getToCity();
		fromDate = afterOta.getFromDate();
		retDate = afterOta.getRetDate();
		tripType = afterOta.getTripType();
		adultNum = afterOta.getAdultNum();
		childNum = afterOta.getChildNum();
		passNum = afterOta.getPassNum();
		isStartLog = afterOta.isStartLog();
		openCaching = afterOta.isOpenCaching();//是否查询缓存
		searchList = new ArrayList<SearchRequestEntity>();
		retSearch = new ArrayList<SearchResponseEntity>();
		searchFromResultSets = new ArrayList<SearchResultSet>();
		searchToResultSets = new ArrayList<SearchResultSet>();
		finalResultSets = new ArrayList<SearchResultSet>();
		platForm = new HashMap<>();
		wholePrice = new HashMap<>();
		resultTripto = new SearchResultTripto();
		resultTripto.setTimeConsuming(new Integer[3]);
		conclusion = new HashMap<>();
		uuid = UUID.randomUUID().toString()+System.currentTimeMillis()+Math.random();
		searchKey = new ArrayList<>();
	}
	
	/**
	 * 1.需求拆解
	 */
	public void DemandDisassembly(){
		//--------log-------------
		SearchLogService.Statistics(platform+"_"+fromCity+"_"+toCity+"_"+tripType+"_"+fromDate+"_"+retDate, 10);//API记录请求数据   平台_出发站_到达站_行程类型_出发时间_到达时间
		SearchLogService.Statistics(platform, 1);//API收集请求数（分平台）
		//1.匹配平台对应航司数据源
		dataSource = ControManage.platformMap.get(platform);
		//2.城市代码转换 + 组装查询需求
		for(String sourceKey:dataSource.keySet()) {
			if(!isGlobalDate(ControManage.wholeDateMap.get(platform+"_"+sourceKey))) continue;
			//加入查询需求
			SearchRequestEntity searchRequestEntity = new SearchRequestEntity();
			searchRequestEntity.setUuid("1");
			
			//日志开启，那么就总受原有的乘客人数
			if(isStartLog) {
				searchRequestEntity.setAdultNum(adultNum);
				searchRequestEntity.setChildNum(childNum);	
			}else {
				//这里需要调整一些  如果传入的乘客人数小于了全局配置  那么采用全局配置
				searchRequestEntity.setAdultNum(adultNum+childNum);
				if(searchRequestEntity.getAdultNum() < dataSource.get(sourceKey)[0]) {
					searchRequestEntity.setAdultNum(dataSource.get(sourceKey)[0]);
				}
			}
			
			searchRequestEntity.setFromDate(ToDate(fromDate));
			if(retDate != null && !retDate.equals("")) {
				searchRequestEntity.setRetDate(ToDate(retDate));	
			}
			searchRequestEntity.setTripType(tripType);
			searchRequestEntity.setUuid(passNum+"");//回调通道地址编号
			String[] airSource = ControManage.airwaysMap.get(sourceKey);
			searchRequestEntity.setAirSource(airSource[0]);//航司数据源编码 如：1
			searchRequestEntity.setSearchId(sourceKey);//航司数据源  如：AK_WEB
			searchRequestEntity.setQueneName(sourceKey);//航司  如:AK
			if(airSource.length == 4 && airSource[3].equals("COUNTRY")) {
				if(ControManage.cityCurrency.containsKey(fromCity)) {
					searchRequestEntity.setCurrency(ControManage.cityCurrency.get(fromCity));	
				}else {
					searchRequestEntity.setCurrency("CNY");
				}
			}else if(airSource.length == 4){
				searchRequestEntity.setCurrency(airSource[3]);
			}else {
				searchRequestEntity.setCurrency("CNY");
			}
			//匹配城市代码转化
			String[] formAirplain = null;
			if(ControManage.cityToAirplainMap.containsKey(sourceKey+fromCity)){
				formAirplain = ControManage.cityToAirplainMap.get(sourceKey+fromCity).split("\\|");
			}
			String[] toAirplain = null;
			if(ControManage.cityToAirplainMap.containsKey(sourceKey+toCity)){
				formAirplain = ControManage.cityToAirplainMap.get(sourceKey+toCity).split("\\|");
			}
			
			if(formAirplain != null && formAirplain.length > 0 && toAirplain == null) {
				//出发有到达没有
				for(String key:formAirplain){
					searchRequestEntity.setFormCity(key);
					searchRequestEntity.setToCity(toCity);
					if(ControManage.airwaysRouteMap.containsKey(searchRequestEntity.getSearchId()+"_"+searchRequestEntity.getFormCity()+"_"+searchRequestEntity.getToCity())) {
						searchList.add(searchRequestEntity);	
					}
				}
			}else if(toAirplain != null && toAirplain.length > 0 && formAirplain == null) {
				//出发没有到达有
				for(String key:toAirplain){
					searchRequestEntity.setToCity(key);
					searchRequestEntity.setFormCity(fromCity);
					if(ControManage.airwaysRouteMap.containsKey(searchRequestEntity.getSearchId()+"_"+searchRequestEntity.getFormCity()+"_"+searchRequestEntity.getToCity())) {
						searchList.add(searchRequestEntity);	
					}
				}
			}else if(toAirplain == null && formAirplain == null) {
				//两个都没有
				searchRequestEntity.setFormCity(fromCity);
				searchRequestEntity.setToCity(toCity);
				if(ControManage.airwaysRouteMap.containsKey(searchRequestEntity.getSearchId()+"_"+searchRequestEntity.getFormCity()+"_"+searchRequestEntity.getToCity())) {
					searchList.add(searchRequestEntity);	
				}
			}else if(formAirplain != null && formAirplain.length > 0 && toAirplain != null && toAirplain.length > 0){
				//全部都需要转化
				for(String key:formAirplain){
					searchRequestEntity.setFormCity(key);
					for(String tokey:toAirplain){
						searchRequestEntity.setToCity(tokey);
						if(ControManage.airwaysRouteMap.containsKey(searchRequestEntity.getSearchId()+"_"+searchRequestEntity.getFormCity()+"_"+searchRequestEntity.getToCity())) {
							searchList.add(searchRequestEntity);	
						}
					}
				}
			} 
		}
		//前期数据组装完成，计算一下时间差
		long endTime = System.currentTimeMillis();    //获取结束时间
		resultTripto.getTimeConsuming()[0] = (int) (endTime - startTime);
	}

	/**
	 * 发起查询，通过openCaching 判断是否走那条线
	 */
	public void SendOutSearchActualTime() {
		if(searchList != null && searchList.size() > 0) {
			if(openCaching) {
				//遍历查询需求并且查询缓存，无缓存的发起缓存
				for (SearchRequestEntity sr : searchList) {
					if(sr.getRetDate() == null) {
						sr.setRetDate("");
					}
					//AK_WEB_KUL_DPS_2_3_20180506_20180520
					String key = sr.getAirSource()+"_"+sr.getFormCity()+"_"+sr.getToCity()+"_"+sr.getTripType();
					key +="_"+sr.getAdultNum()+"_"+sr.getFromDate()+"_"+sr.getRetDate();
					//1.第二步检查缓存中是否存在
					SearchResponseEntity sre = SearchEntrustService.QueryRedis(key);
					if(sre != null) {
						retSearch.add(sre);
						continue;
					}
					//2.判断是否发起过查询，如果发起过拦截
					if(!SearchEntrustService.addSearchEntrust(key, this.uuid, this)) {
						SearchEntrustService.SendOutSeaarch(sr); 
					}
					searchKey.add(key);
					conclusion.put(key, false);
					System.out.println("加入等待队列："+key);
				}
			}else {
				//实现实时查询
				for (SearchRequestEntity sr : searchList) {
					SearchResponseEntity  srEntity = SpiderSearchUtils.verify(sr);
					srEntity.setSuitAge(SearchEntrustService.sdf.format(new Date()));//查询时间
					retSearch.add(srEntity);
					String airName = srEntity.getSearchId();//获取航司编码
					String airSource = srEntity.getFromSegments().get(0).getAirSourceName();
					SearchLogService.Statistics(airSource, 4);//4.爬虫回调收集
					//组装KEY  AK_WEB_KUL_DPS_2_3_20180506_20180520
					if(srEntity.getRetDate() == null) {
						srEntity.setRetDate("");
					}
					String key = airSource+"_"+srEntity.getFormCity()+"_"+srEntity.getToCity()+"_"+srEntity.getTripType()+"_"+srEntity.getAdultNum()+"_"+srEntity.getFromDate()+"_"+srEntity.getRetDate();
					SearchEntrustService.addRedis(airName,key,srEntity);//入redis
				}
			}
		}
		if(openCaching) {
			ThisStop();
		}
	}
	
	/**
	 * 查询等待模式
	 */
	public void ThisStop() {
		for(int i = 0; i < 25;i++) {
			try {
				if(conclusion == null || conclusion.size() == 0) {
					break;
				}
				Thread.sleep(500);//0.5s检查一次
				for (String concKey : conclusion.keySet()) {
					if(!conclusion.get(concKey)) break;
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		
		for(String key:searchKey) {
			SearchEntrustService.deleteSearchEntrust(key, uuid);
		}
	}
	
	/**
	 * 查询结果回调
	 * @param key
	 * @param result
	 */
	public void ResultCallBack(String key,SearchResponseEntity result) {
		retSearch.add(result);
		conclusion.remove(key);
	}
	
	/**
	 * 加载全局计算模型
	 */
	public void LoadTheGlobal(){
		//查询已经完成，计算一下时间差
		long endTime = System.currentTimeMillis();    //获取结束时间
		resultTripto.getTimeConsuming()[1] = (int) (endTime - startTime);
		
		//5.加载全局价格计算(平台)
		if(ControManage.WholePlatformPrice.containsKey(platform)){
			platForm = ControManage.WholePlatformPrice.get(platform);
		}
		
		//6.加载全局调价(平台航司)
		for(String airName:dataSource.keySet()) {
			String wholeKey = platform+"_"+airName;
			if(!ControManage.wholePriceMap.containsKey(wholeKey)) continue;
			//Key开始日期,结束日期
			Map<String,Map<String,Integer[]>> wpMap = ControManage.wholePriceMap.get(wholeKey);
			for(String dateKey:wpMap.keySet()) {
				// date 20180701,20181130  fromDate 
				String[] dss = dateKey.split(",");
				if(Integer.valueOf(dss[0]) < Integer.valueOf(fromDate) && Integer.valueOf(fromDate) <= Integer.valueOf(dss[1])) {
					Map<String,Integer[]> childMap = wpMap.get(dateKey);
					for (String key : childMap.keySet()) {
						wholePrice.put(airName+"_"+key, childMap.get(key));
					}
				}
			}
		}
	}
	
	/**
	 * 2.解析
	 */
	public void Analysis(){
		for (SearchResponseEntity sre : retSearch) {
			String airName = sre.getSearchId();//如：AK
			String policyKey = "";
			if(sre.getFromSegments() != null && sre.getFromSegments().size() > 0){
				List<AirlineInfomation> aList = sre.getFromSegments();
				for (AirlineInfomation ainfo : aList) {
					String aKey = platform+"_"+airName+"_"+ainfo.getFormCity()+"_"+ainfo.getToCity();
					if(!policyKey.equals(aKey)){
						policyKey = aKey;
					}
					List<SearchResultSet> msg = AnalysisSearchRet(airName,ainfo,sre.getSuitAge());//进行处理
					searchFromResultSets.addAll(msg);
				}
			}
			if(sre.getRetSegments() != null && sre.getRetSegments().size() > 0){
				List<AirlineInfomation> aList = sre.getRetSegments();
				for (AirlineInfomation ainfo : aList) {
					String aKey = platform+"_"+airName+"_"+ainfo.getFormCity()+"_"+ainfo.getToCity();
					if(!policyKey.equals(aKey)){
						policyKey = aKey;
					}
					List<SearchResultSet> msg = AnalysisSearchRet(airName,ainfo,sre.getSuitAge());//进行处理
					searchToResultSets.addAll(msg);
				}
			}
		}
	}
	
	/**
	 * 2.1解析结果
	 * @param airName
	 * @param policyList
	 * @param ainfo
	 */
	public List<SearchResultSet> AnalysisSearchRet(String airName,AirlineInfomation ainfo,String sharchDatetime) {
		List<SearchResultSet> restSet = new ArrayList<>(); 
		SearchResultSet resultSet = new SearchResultSet();
		resultSet.setSharchDatetime(sharchDatetime);//缓存时间
		resultSet.setFromDate(ainfo.getFromDate());            	//2018-06-12, 起飞时间
		resultSet.setDurationTotal(ainfo.getDurationTotal());   //70, 飞行总时长 分钟
		resultSet.setToCity(ainfo.getToCity());              	//HDY,  到达机场三字码
		resultSet.setAirName(airName);             				//航司 如：AK
		resultSet.setAirSourceName(ainfo.getAirSourceName());   //航司数据源名称 如：AK_WEB
		resultSet.setFormCity(ainfo.getFormCity());             //KUL, 出发机场三字码
		resultSet.setRetDate(ainfo.getRetDate());               //2018-06-20, 返程时间
		List<RouteInfoEntity> routeInfoEntities = ainfo.getRouteInfoEntities();
		if(routeInfoEntities != null && routeInfoEntities.size() > 0) {
			List<SearchResultSetSegments> segmentsList = new ArrayList<>();
			for (RouteInfoEntity route : routeInfoEntities) {
				SearchResultSetSegments seg = new SearchResultSetSegments();
				seg.setArrAirport(route.getArrAirport()); // HDY,  到达机场三字码
				seg.setDepAirport(route.getDepAirport()); // KUL, 出发机场三字码
				seg.setArrTime(route.getArrTime()); // 201806121520  出发时间
				seg.setDepTime(route.getDepTime()); // 201806121510, 到达时间
				seg.setCarrier(route.getCarrier()); // FD, 航司
				seg.setFlightNumber(route.getFlightNumber()); // FD461, 航班号
				seg.setDuration(route.getDuration()); //70, 飞行时长 分钟
				seg.setCabin(route.getCabin()); // E,舱位
				seg.setAircraftCode(route.getAircraftCode()); // 320, 机型
				segmentsList.add(seg);
			}
			resultSet.setFromSegments(segmentsList);//去程航班信息
		}
		List<AirlinePriceInfo> airlinePriceInfos = ainfo.getAirlinePriceInfos();
		resultSet.setCurrency(ainfo.getCurrency());            //CNY, 币种
		resultSet.setOrigCurrency(ainfo.getCurrency());//辅助：记录返回原始币种
		//判断是否需要汇率计算
		BigDecimal exchangeRate = null; 
		if(!ainfo.getCurrency().equals("CNY")) {
			//判断是否需要采用航司汇率 airName
			String currKey = "CNY_"+ainfo.getCurrency();
			if(ControManage.exchangeRateMap.containsKey(airName)) {
				if(ControManage.exchangeRateMap.get(airName).containsKey(currKey)) {
					exchangeRate = ControManage.exchangeRateMap.get(airName).get(currKey);
					resultSet.setCurrency("CNY");
				}
			}else {
				if(ControManage.UnionPayExchangeRateMap.containsKey(currKey)) {
					exchangeRate = ControManage.UnionPayExchangeRateMap.get(currKey);
					resultSet.setCurrency("CNY");
				}
			}
			resultSet.setOrigCurrRate(currKey+"="+exchangeRate);//辅助：拿取汇率KEY=汇率
		}
		
		for (AirlinePriceInfo aprice : airlinePriceInfos) {
			//辅助：记录原始价格
			resultSet.setOrigGoAdultPrice(aprice.getAdultPrice());
			resultSet.setOrigGoAdultTax(aprice.getAdultTax());
			resultSet.setOrigGoChildPrice(aprice.getChildPrice());
			resultSet.setOrigGoChildTax(aprice.getChildTax());
			
			resultSet.setAdultPrice(aprice.getAdultPrice().add(aprice.getAdultTax()));//369.07, 成人税费+成人票价
			
			
			
			if(exchangeRate != null){
				resultSet.setAdultPrice(resultSet.getAdultPrice().multiply(exchangeRate));     //369.07, 成人税费
			}
			resultSet.setCabin(aprice.getCabin());               //VF,  舱位等级
			resultSet.setCabinClass(aprice.getCabinclass());     //1, 舱位等级编码
			resultSet.setCabinName(aprice.getCabinName());       //超值套餐 舱位等级名称
			resultSet.setMaxSeats(aprice.getMaxSeats());         //5,  最大座位数
			//辅助：记录人民币价格 转换后的价格
			resultSet.setOrigGoPrice(resultSet.getAdultPrice());
			
			restSet.add((SearchResultSet)resultSet.clone());
			//判断是否支持急速出票
			if(aprice.getCabin().equals("EC") && dataSource.get(airName)[1] == 1) {
				resultSet.setCabinClass("8");     //1, 舱位等级编码
				restSet.add((SearchResultSet)resultSet.clone());
			}
		}
		return restSet;
	}
	
	/**
	 * 3.匹配政策
	 */
	public void MatchingPolicy(){
		//去程政策匹配
		if(searchFromResultSets != null && searchFromResultSets.size() > 0){
			List<PolicyEntity> policyList = null;
			String policyKey = "";
			for (SearchResultSet sfrs : searchFromResultSets) {
				String aKey = platform+"_"+sfrs.getAirName()+"_"+sfrs.getFormCity()+"_"+sfrs.getToCity();
				if(!ControManage.policyMap.containsKey(aKey))continue;
				if(!policyKey.equals(aKey)) {
					policyKey = aKey;
					policyList = ControManage.policyMap.get(aKey);
				}
				int fromDate = Integer.valueOf(sfrs.getFromDate().replaceAll("-", ""));
				//这里做价格计算  首先验证日期是否符合规定 在区间内
				if(policyList != null && policyList.size() > 0){
					MatchingPolicyCount(policyList, fromDate, sfrs);//4.1匹配政策 进行价格计算
				}
			}
		}
		//返程政策匹配
		if(searchToResultSets != null && searchToResultSets.size() > 0){
			List<PolicyEntity> policyList = null;
			String policyKey = "";
			for (SearchResultSet sfrs : searchToResultSets) {
				String aKey = platform+"_"+sfrs.getAirName()+"_"+sfrs.getFormCity()+"_"+sfrs.getToCity();
				if(!ControManage.policyMap.containsKey(aKey))continue;
				if(!policyKey.equals(aKey)) {
					policyKey = aKey;
					policyList = ControManage.policyMap.get(aKey);
				}
				int fromDate = Integer.valueOf(sfrs.getFromDate().replaceAll("-", ""));
				//这里做价格计算  首先验证日期是否符合规定 在区间内
				if(policyList != null && policyList.size() > 0){
					MatchingPolicyCount(policyList, fromDate, sfrs);//4.1匹配政策 进行价格计算
				}
			}
		}
	}
	
	/**
	 * 4.1匹配政策 进行价格计算
	 * @param policyList
	 * @param fromDate
	 * @param sfrs
	 */
	public void MatchingPolicyCount(List<PolicyEntity> policyList,int fromDate,SearchResultSet sfrs) {
		for (PolicyEntity pe : policyList) {
			//前期已经通过平台航司航线进行分组。
			//第一步判断 是否在日期区间内  fromDateStart  fromDateEnd
			if(pe.fromDateStart != null && pe.fromDateEnd != null) {
				if(pe.fromDateStart > fromDate || pe.fromDateEnd < fromDate) continue;
			}
			//第二步判断 是否排除在日期区间内
			if(pe.excludeFromDateStart != null && pe.excludeFromDateEnd != null && (pe.excludeFromDateStart+pe.excludeFromDateEnd) > 4000) {
				if(pe.excludeFromDateStart < fromDate || pe.excludeFromDateEnd > fromDate) continue;
			}
			//第三步判断 产品类型
			boolean isok = true;
			if(pe.cabinScope != Integer.valueOf(sfrs.getCabinClass())) {
				isok = false;
				if(pe.cabinScope == 9)isok = true;
			}
			if(!isok) continue;
			//第四步判断 价格区间拦截
			float price = sfrs.getAdultPrice().floatValue();
			if(pe.lowPrice > price || pe.upPrice < price) continue;
			//第五步判断 是否航班号限制
			if(pe.flightNumberLimitType != null && pe.flightNumberLimitType == 1) {
				boolean isSeg = true;
				for (SearchResultSetSegments seg : sfrs.getFromSegments()) {
					if(!pe.flightNumberLimit.contains(seg.getFlightNumber())) {
						isSeg = false;
						break;
					}
				}
				if(!isSeg) continue;
			}
			//第六步判断 价格匹配类型 0.固定价格  1.加减价
			if(pe.adjustType != null && pe.adjustType == 0 && pe.finalPrice > 0) {
				sfrs.setAdultPrice(new BigDecimal(pe.finalPrice));
			}
			//百分比 成人价 
			if(pe.adjustRadio > 0) sfrs.setAdultPrice(sfrs.getAdultPrice().add(sfrs.getAdultPrice().multiply(new BigDecimal(pe.adjustRadio).divide(new BigDecimal(100)))));
			//加减价计算 成人价 
			sfrs.setAdultPrice(sfrs.getAdultPrice().add(new BigDecimal(pe.addOrSubtractMoney)));
			//这里表示已经命中政策了
			sfrs.setOrigGoPolicy(JSON.toJSONString(pe));//辅助：记录命中政策对象
			break;
		}
	}
	
	/**
	 * 4.加工单程往返
	 */
	public void MachiningType(){
		
		if(searchFromResultSets != null && searchFromResultSets.size() > 0 && searchToResultSets != null && searchToResultSets.size() > 0) {
			//往返
			for (SearchResultSet sfrs : searchFromResultSets) {
				if(!sfrs.getCabin().equals("EC")) continue;
				for (SearchResultSet strs : searchToResultSets) {
					if(!strs.getCabin().equals("EC") || !sfrs.getCabinClass().equals(strs.getCabinClass())) continue;
					//辅助：同步价格与政策
					SearchResultSet sfrsTrue = (SearchResultSet)sfrs.clone();
					sfrsTrue.setOrigRetAdultPrice(strs.getOrigGoAdultPrice());
					sfrsTrue.setOrigRetAdultTax(strs.getOrigGoAdultTax());
					sfrsTrue.setOrigRetChildPrice(strs.getOrigGoChildPrice());
					sfrsTrue.setOrigRetChildTax(strs.getOrigGoChildTax());
					sfrsTrue.setOrigRetPolicy(strs.getOrigGoPolicy());
					sfrsTrue.setOrigRetPrice(strs.getOrigGoPrice());
					sfrsTrue.setRetSegments(strs.getFromSegments());
					sfrsTrue.setRetDate(strs.getFromDate());
					//成人价格累计
					sfrsTrue.setAdultPrice(sfrsTrue.getAdultPrice().add(strs.getAdultPrice()));
					//最大座位数比较，谁的位置数低走谁
					if(sfrsTrue.getMaxSeats() > strs.getMaxSeats()) {
						sfrsTrue.setMaxSeats(strs.getMaxSeats());
					}
					finalResultSets.add(sfrsTrue);
				}
			}
		}else {
			//单程
			finalResultSets = searchFromResultSets;
		}
	}
	
	/**
	 * 5.全局调价 平台加减价 航司平台加减价
	 */
	public void GlobalPrice(){
		//KEY=产品类型   Map<String,Integer[]> platForm;//平台加减价
		//KEY=航司_产品类型  Map<String,Integer[]> wholePrice;//航司平台加减价

		for (SearchResultSet frs : finalResultSets) {
			//0.同步儿童价格
			frs.setChildPrice(frs.getAdultPrice());
			String priKey = frs.getAirName()+"_"+frs.getCabinClass();
			//1.平台航司加减价
			if(wholePrice != null && wholePrice.size() > 0 && !wholePrice.containsKey(priKey)) {
				//全局不能干扰 急速出票
				if(!frs.getCabinClass().equals("8")) {
					priKey = frs.getAirName()+"_9";//未匹配的具体套餐使用全部模式
				}
			}
			//先比例计算 然后加减钱 计算公式 = 原价+(原价*调价比例+加减价)  [成人加减价,儿童加减价,成人调价比例,儿童调价比例]
			if(wholePrice != null && wholePrice.size() > 0 && wholePrice.containsKey(priKey)){
				Integer[] nums =  wholePrice.get(priKey);
				if(nums.length == 4){
					frs.setAdultPrice(CalculatePrice(frs.getAdultPrice(),nums[2],nums[0]));
					frs.setChildPrice(CalculatePrice(frs.getChildPrice(),nums[3],nums[1]));
				}
				frs.setOrigWholePrice(priKey+"="+JSON.toJSONString(nums));//辅助：记录平台航司加减价
			}
			//2.平台自我加减价
			String pFormKey = frs.getCabinClass();
			if(platForm != null && platForm.size() > 0 && !platForm.containsKey(pFormKey)) {
				//全局不能干扰 急速出票
				if(!frs.getCabinClass().equals("8")) {
					pFormKey = "9";//未匹配的具体套餐使用全部模式
				}
			}
			if(platForm != null && platForm.size() > 0 && platForm.containsKey(pFormKey)) {
				//先比例计算 然后加减钱 计算公式 = 原价+(原价*调价比例+加减价)  [成人加减价,儿童加减价,成人调价比例,儿童调价比例]
				Integer[] nums =  platForm.get(pFormKey);
				if(nums.length == 4) {
					frs.setAdultPrice(CalculatePrice(frs.getAdultPrice(),nums[2],nums[0]));
					frs.setChildPrice(CalculatePrice(frs.getChildPrice(),nums[3],nums[1]));
				}
				frs.setOrigPlatForm(pFormKey+"="+JSON.toJSONString(nums));//辅助：记录平台自我加减价
			}
			//3.拆分票价
			frs.setAdultTax(frs.getAdultPrice().multiply(new BigDecimal(0.1)).setScale(0, BigDecimal.ROUND_HALF_UP));
			frs.setAdultPrice(frs.getAdultPrice().multiply(new BigDecimal(0.9)).setScale(0, BigDecimal.ROUND_HALF_UP));
			frs.setChildTax(frs.getChildPrice().multiply(new BigDecimal(0.1)).setScale(0, BigDecimal.ROUND_HALF_UP));
			frs.setChildPrice(frs.getChildPrice().multiply(new BigDecimal(0.9)).setScale(0, BigDecimal.ROUND_HALF_UP));
		}
	}
	
	
	/**
	 * 价格计算 计算公式 = 原价+(原价*调价比例+加减价)
	 * @param original 原价
	 * @param ratio 调价比例  比例需要先/100
	 * @param addAndAubtract 加减价
	 * @return
	 */
	public BigDecimal CalculatePrice(BigDecimal original,Integer ratio,Integer addAndAubtract) {
		BigDecimal ratioBig = new BigDecimal(ratio);
		ratioBig = ratioBig.divide(new BigDecimal(100));//比例需要先/100
		BigDecimal addAndAubtractBig = new BigDecimal(addAndAubtract);
		return original.add(original.multiply(ratioBig).add(addAndAubtractBig));
	}
	
	/**
	 * 全局日期拦截
	 * @param days
	 * @return
	 */
	public boolean isGlobalDate(Integer days){
		if(Integer.valueOf(fromDate) < ControManage.currentDate+days) {
			return false;
		}
		return true;
	}

	/**
	 * 日期转换
	 * @param date yyyyMMdd
	 * @return yyyy-MM-dd
	 */
	public String ToDate(String date) {
		return date.substring(0, 4)+"-"+date.substring(4, 6)+"-"+date.substring(6, 8);
	}
	
	/**
	 * 7.获取最终查询结果集
	 * @return
	 */
	public SearchResultTripto getFinalResult() {
		//判断是否开启了记录日志模式
		if(isStartLog) {
			//前期收集请求参数，匹配数据源   发送组装数据记录
			resultTripto.setStartLog(isStartLog);
			resultTripto.setRetSearch(retSearch);//查询返回原始结果
			resultTripto.setSearchRequestAfterOta(afterOta);//API请求数据
			resultTripto.setSearchList(searchList);//查询任务拆解  请求结果集
		}
		resultTripto.setSearchResultSets(finalResultSets);//结果集
		//最后的时间
		long endTime = System.currentTimeMillis();    //获取结束时间
		resultTripto.getTimeConsuming()[2] = (int) (endTime - startTime);
		return resultTripto;
	}
	
	public static void main(String[] args) { 
		
	}
}
