package com.tripto.bjlt.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.entity.SearchResponseEntity;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

/**
 * Redis工具类操作
 * @ClassName: RedisUtil 
 * @Description: TODO(这里用一句话描述这个类的作用) 
 * @author zhangqifei 
 * @date 2018年4月3日 下午12:19:59
 */
public class RedisUtil {

	//Redis服务器IP
    private static String ADDR = "101.200.40.164";
    //Redis的端口号
    private static int PORT = 12301;
    //访问密码
    private static String AUTH = "tripto!@#$1803";
    //可用连接实例的最大数目，默认值为8；
    //如果赋值为-1，则表示不限制；如果pool已经分配了maxActive个jedis实例，则此时pool的状态为exhausted(耗尽)。
    private static int MAX_ACTIVE = 1000;
    //控制一个pool最多有多少个状态为idle(空闲的)的jedis实例，默认值也是8。
    private static int MAX_IDLE = 100;
    //等待可用连接的最大时间，单位毫秒，默认值为-1，表示永不超时。如果超过等待时间，则直接抛出JedisConnectionException；
    private static int MAX_WAIT = 10000;
    private static int TIMEOUT = 10000;
    //在borrow一个jedis实例时，是否提前进行validate操作；如果为true，则得到的jedis实例均是可用的；
    private static boolean TEST_ON_BORROW = true;
    private static JedisPool jedisPool = null;

    /**
     * 初始化Redis连接池
     */
    public static void Init(String h,int p,String a,int max_active,int max_idle,int max_wait,int timeout){
        try {
        	ADDR = h;
        	PORT = p;
        	AUTH = a;
        	MAX_ACTIVE = max_active;
        	MAX_IDLE = max_idle;
        	MAX_WAIT = max_wait;
        	TIMEOUT = timeout;
            JedisPoolConfig config = new JedisPoolConfig();
            config.setMaxTotal(MAX_ACTIVE);
            config.setMaxIdle(MAX_IDLE);
            config.setMaxWaitMillis(MAX_WAIT);
            config.setTestOnBorrow(TEST_ON_BORROW);
            jedisPool = new JedisPool(config, ADDR, PORT, TIMEOUT, AUTH);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取Jedis实例
     * @return
     */
    public synchronized static Jedis getJedis() {
        try {
            if (jedisPool != null) {
                Jedis resource = jedisPool.getResource();
                return resource;
            } else {
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 释放jedis资源
     * @param jedis
     */
    public static void returnResource(final Jedis jedis) {
        if (jedis != null) {
            jedisPool.returnResource(jedis);
        }
    }
	
    /**
     * 入Redis
     * @param key   键
     * @param value 值
     * @param duration 过期时间(秒)
     */
    public static void addRedis(String key,String value,int duration) {
    	Jedis jedis = getJedis();
    	jedis.set(key, value);
    	jedis.expire(key,duration);
    	returnResource(jedis);
    }
    
    /**
     * 读取Redis
     * @param key
     * @return
     */
    public static String getRedis(String key) {
    	Jedis jedis = getJedis();
    	String info = jedis.get(key);
    	returnResource(jedis);
    	return info;
    }
    
    /**
     * 统计当前缓存数量
     * @return
     */
    public static int countKey() {
    	int num = 0 ;
    	Jedis jedis = getJedis();
    	num = jedis.keys("*").size();
    	returnResource(jedis);
    	return num;
    }
    
    public static void main(String[] args) {
    	String sr = RedisUtil.getRedis("AK_WEB_KUL_DPS_2_2_2018-04-21_2018-04-23");
    	System.out.println(sr);
    	SearchResponseEntity responseEntity = JSONObject.toJavaObject(JSON.parseObject(sr), SearchResponseEntity.class);
    	System.out.println(responseEntity);
	}
}
